#version 330 core
// Interpolated values from the vertex shaders
in vec2 UV;


in vec3 Position_worldspace;
in vec3 Normal_cameraspace;
in vec3 EyeDirection_cameraspace;
in vec3 LightDirection_cameraspace;

in vec4 pos;
in vec4 prev_pos;

// Ouput data
layout(location = 0) out vec3 color;
layout(location = 1) out vec3 velocity;

// Values that stay constant for the whole mesh.
uniform sampler2D myTextureSampler;
uniform sampler2DShadow shadowMap1;
uniform sampler2DShadow shadowMap2;
uniform sampler2DShadow shadowMap3;
uniform  mat4 ShadowCoord1;
uniform  mat4 ShadowCoord2;
uniform  mat4 ShadowCoord3;
uniform  vec4 far_d;
uniform bool showFrustums; 

in vec4 vPos;
in float depth;

uniform vec3 LightPosition_worldspace;

vec2 poissonDisk[16] = vec2[]( 
   vec2( -0.94201624, -0.39906216 ), 
   vec2( 0.94558609, -0.76890725 ), 
   vec2( -0.094184101, -0.92938870 ), 
   vec2( 0.34495938, 0.29387760 ), 
   vec2( -0.91588581, 0.45771432 ), 
   vec2( -0.81544232, -0.87912464 ), 
   vec2( -0.38277543, 0.27676845 ), 
   vec2( 0.97484398, 0.75648379 ), 
   vec2( 0.44323325, -0.97511554 ), 
   vec2( 0.53742981, -0.47373420 ), 
   vec2( -0.26496911, -0.41893023 ), 
   vec2( 0.79197514, 0.19090188 ), 
   vec2( -0.24188840, 0.99706507 ), 
   vec2( -0.81409955, 0.91437590 ), 
   vec2( 0.19984126, 0.78641367 ), 
   vec2( 0.14383161, -0.14100790 ) 
);


float calcVisibilityWithPoisson(sampler2DShadow map, vec4 ShadowCoord){

	float visibility = 0.0f;
	vec2 size = textureSize(map,0);

	for (int i=0;i<16;i++){
		
			int index = i;
			visibility += textureProj( map, vec4(ShadowCoord.xy + poissonDisk[index]/(size.x/0.9),  ShadowCoord.z, ShadowCoord.w ) );
		}
	return	visibility /= 16;
}

void main(){


	  vec2 a = (pos.xy / pos.w) * 0.5 + 0.5;
      vec2 b = (prev_pos.xy / prev_pos.w) * 0.5 + 0.5;
      velocity = vec3( (a - b) , 1);

	// Light emission properties
	vec3 LightColor = vec3(1,1,1);
	float LightPower = 1.0f;
	
	// Material properties
	vec3 MaterialDiffuseColor = texture2D( myTextureSampler, UV ).rgb;
	vec3 MaterialAmbientColor = vec3(0.3,0.3,0.3) * MaterialDiffuseColor;
	vec3 MaterialSpecularColor = vec3(0.1,0.1,0.1);

	// Distance to the light
	//float distance = length( LightPosition_worldspace - Position_worldspace );

	// Normal of the computed fragment, in camera space
	vec3 n = normalize( Normal_cameraspace );
	// Direction of the light (from the fragment to the light)
	vec3 l = normalize( LightDirection_cameraspace );
	// Cosine of the angle between the normal and the light direction, 
	// clamped above 0
	//  - light is at the vertical of the triangle -> 1
	//  - light is perpendiular to the triangle -> 0
	//  - light is behind the triangle -> 0
	float cosTheta = clamp( dot( n,l ), 0,1 );
	
	// Eye vector (towards the camera)
	vec3 E = normalize(EyeDirection_cameraspace);
	// Direction in which the triangle reflects the light
	vec3 R = reflect(-l,n);
	// Cosine of the angle between the Eye vector and the Reflect vector,
	// clamped to 0
	//  - Looking into the reflection -> 1
	//  - Looking elsewhere -> < 1
	float cosAlpha = clamp( dot( E,R ), 0,1 );
	
	vec4 ShadowCoord;
	float visibility = 0.0f;
	vec3 coColor;
	// find the appropriate depth map to look up in based on the depth of this fragment
	if(depth < far_d.x){

		coColor = vec3(1,0,0);
		ShadowCoord = ShadowCoord1*vPos;
		visibility = calcVisibilityWithPoisson(shadowMap1,ShadowCoord );
	
	}else if(depth < far_d.y){
		ShadowCoord = ShadowCoord2*vPos;
		coColor = vec3(0,1,0);
		visibility = calcVisibilityWithPoisson(shadowMap2,ShadowCoord );

		
	}else if(depth < far_d.z){
		ShadowCoord = ShadowCoord3*vPos;
		
		coColor = vec3(0,0,1);

		visibility = calcVisibilityWithPoisson(shadowMap3,ShadowCoord );

	}
	

		color = 
		// Ambient : simulates indirect lighting
		MaterialAmbientColor +
		// Diffuse : "color" of the object
		visibility * MaterialDiffuseColor * LightColor * LightPower * cosTheta+
		// Specular : reflective highlight, like a mirror
		visibility * MaterialSpecularColor * LightColor * LightPower * pow(cosAlpha,5) ;

		if(showFrustums)
			color += coColor;



}

