#version 330 core
// Input vertex data, different for all executions of this shader.
layout(location = 0) in vec3 vertexPosition_modelspace;
layout(location = 1) in vec2 vertexUV;
layout(location = 2) in vec3 vertexNormal_modelspace;

// Output data ; will be interpolated for each fragment.
out vec2 UV;
out vec3 Position_worldspace;
out vec3 Normal_cameraspace;
out vec3 EyeDirection_cameraspace;
out vec3 LightDirection_cameraspace;


// Values that stay constant for the whole mesh.
uniform mat4 viewMatrix;
uniform mat4 modelMatrix;
uniform mat4 projectionMatrix;
uniform mat4 invertedTransposedModelViewMatrix;
uniform mat4 prevVPMatrix;
uniform mat4 prevModelMatrix;

// this should not change
uniform vec3 LightPosition_worldspace;

out vec4 vPos;
out float depth;

out vec4 pos;
out vec4 prev_pos;

void main(){

	pos =  projectionMatrix *  viewMatrix * modelMatrix * vec4(vertexPosition_modelspace,1);
	prev_pos = prevVPMatrix * prevModelMatrix * vec4(vertexPosition_modelspace,1);
	gl_Position = pos;
	
	vPos = modelMatrix * vec4(vertexPosition_modelspace,1);
	depth = gl_Position.z;
	
	
	// Position of the vertex, in worldspace : M * position
	Position_worldspace = (modelMatrix * vec4(vertexPosition_modelspace,1)).xyz;
	
	// Vector that goes from the vertex to the camera, in camera space.
	// In camera space, the camera is at the origin (0,0,0).
	vec3 vertexPosition_cameraspace = (viewMatrix * modelMatrix * vec4(vertexPosition_modelspace,1)).xyz;
	EyeDirection_cameraspace = vec3(0,0,0) - vertexPosition_cameraspace;

	// Vector that goes from the vertex to the light, in camera space. M is ommited because it's identity.
	vec3 LightPosition_cameraspace = (viewMatrix * vec4(LightPosition_worldspace,1)).xyz;
	LightDirection_cameraspace = LightPosition_cameraspace + EyeDirection_cameraspace;
	
	// Normal of the the vertex, in camera space
	Normal_cameraspace = ( viewMatrix * modelMatrix* vec4(vertexNormal_modelspace,0)).xyz; // Only correct if ModelMatrix does not scale the model ! Use its inverse transpose if not.
	

	// UV of the vertex. No special space for this one.
	UV = vertexUV;
}

